(function () {
	function loadScriptList(scriptList) {
		var oHead = document.head || document.getElementsByTagName('head')[0];

		var randomizer = Math.random().toString(10).substring(2, 8);

		return new Promise(function (resolve, reject) {

			setTimeout(function () {
				var error = new Error();
				error.Code = ErrorCodes.StaptupScriptTimeout;
				reject(error);
			}, 20000);

			var onAfterLoadSingle = _.after(scriptList.length, resolve);
			_.each(scriptList, loadSingleScript);

			function loadSingleScript(script) {
				var sc = document.createElement('script');

				sc.onload = onAfterLoadSingle;
				sc.onerror = function () {
					var error = new Error('error load: ' + script);
					error.Code = ErrorCodes.StartupScriptLoad;
					reject(error);
				};

				sc.setAttribute('type', 'text/javascript');
				sc.setAttribute('src', script + '?rnd=' + randomizer);

				oHead.appendChild(sc);
			}
		});
	}

	function loadFrame() {
		if (!window.application) {
			var error = new Error();
			error.Code = ErrorCodes.NoApplication;
			throw error;
		} else {
			application.run(4); // version ?
		}
	}

	function onDeviceReady() {
		// если соединения нет никакого, то об этом говорим сразу
		if (navigator.connection && navigator.connection.type === 'none') {
			showConnectionErrorMessage(ErrorCodes.NoConnection);
			location.reload();
			return;
		}

		// праячем status bar
		AndroidFullScreen.immersiveMode(function () {
			console.log('Immersive mode enabled');
		}, function (err) {
			console.log(err);
		});

		loadScriptList(scriptList)
			.then(loadFrame)
			.catch(function (error) {
				if (error) {
					if (error.Code) {
						showConnectionErrorMessage(error.Code);
					} else {
						if (error.stack) {
							window.fabric.Crashlytics.setStringValueForKey(error.stack, 'stack');
						}
						window.fabric.Crashlytics.sendNonFatalCrash(error.message);
					}
				}

				location.reload();
			});
	}

	function showConnectionErrorMessage(step) {
		window.fabric.Answers.sendCustomEvent('startupError', {
			code: String(step)
		});

		navigator.globalization.getLocaleName(function (data) {
			var locale = data.value.split('-')[0];
			showMessageForLocale(step, locale);
		}, function () {
			showMessageForLocale(step);
		});

		function showMessageForLocale(step, locale) {
			if (!connectionErrorText[locale]) {
				locale = 'en';
			}

			var msg = _.template(connectionErrorText[locale])({errorCode: step});
			alert(msg);
		}
	}

	var connectionErrorText = {
		ru: 'Невозможно запустить приложение, проверьте подключение к интернету! (код ошибки: <%= errorCode %>)',
		en: 'Unable to start the application, check the the Internet connection! (error code: <%= errorCode %>)',
		jp: 'アプリケーションを起動することができません、インターネット接続を確認！'
	};

	var ErrorCodes = {
		NoConnection: 10,
		StaptupScriptTimeout: 11,
		StartupScriptLoad: 12,
		NoApplication: 13
	};

	document.addEventListener('deviceready', onDeviceReady, false);
})();
